#include <SDL.h>
#include <GL/gl.h>
#include <assert.h>
#include <stdexcept>
#include "gl.hpp"
#include "module.hpp"
#ifndef NDEBUG
#include <iostream>
#endif

namespace oiran { namespace linda {


	namespace {


		const Uint32 one_second = 1000;


	} // unnamed namespace.


	const Uint32 module_t::default_framerate = 60;


	module_t::module_t() :
		M_screen_width(0),
		M_screen_height(0),
		M_framerate(0),
		M_setted_framerate(default_framerate),
		M_last_update_time(0),
		M_last_calculate_time(0),
		M_frame_count(0)
	{
		if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO) < 0)
			throw std::runtime_error(SDL_GetError());
#ifndef NDEBUG
		std::cout << "initialized SDL." << std::endl;
#endif
	}

	module_t::~module_t()
	{
		SDL_Quit();
#ifndef NDEBUG
		std::cout << "shutted down SDL." << std::endl;
#endif
	}

	void
	module_t::screen(int width, int height)
	{
		assert(width > 0 && height > 0 && "invalid screen size.");

		if(!SDL_SetVideoMode(width, height, 0, SDL_OPENGL))
			throw std::runtime_error(SDL_GetError());

		glViewport(0, 0, width, height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0, width, height, 0, -1, 1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		check_gl_error();

		M_screen_width = width;
		M_screen_height = height;
	}

	bool
	module_t::handle_event()
	{
		if(M_last_update_time == 0)
		{
			M_last_update_time = M_last_calculate_time = SDL_GetTicks();
#ifndef NDEBUG
			std::cout << "start game loop." << std::endl;
#endif
		}

		check_gl_error();

		return _handle_event();
	}

	void
	module_t::wait()
	{
		++M_frame_count;
		_wait();
		_calculate_framerate();
	}

	void
	module_t::title(const char* value)
	{
		SDL_WM_SetCaption(value, 0);
	}

	void
	module_t::framerate(Uint32 value)
	{
		M_setted_framerate = value;
	}

	bool
	module_t::_handle_event()
	{
		SDL_Event event;
		while(SDL_PollEvent(&event))
		{
			if(event.type == SDL_QUIT)
				return false;
			if(event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)
				return false;
		}
		return true;
	}

	void
	module_t::_wait()
	{
		Uint32 now = SDL_GetTicks();
		if(M_setted_framerate)
		{
			Uint32 interval = one_second / M_setted_framerate;
			Uint32 elapsed = now - M_last_update_time;
			if(interval > elapsed)
				SDL_Delay(interval - elapsed);
			M_last_update_time += interval;
		}
		else
			M_last_update_time = now;
	}

	void
	module_t::_calculate_framerate()
	{
		Uint32 now = SDL_GetTicks();
		Uint32 elapsed = now - M_last_calculate_time;
		if(elapsed >= one_second)
		{
			M_framerate = 1.0f * M_frame_count * one_second / elapsed;
			M_frame_count = 0;
			M_last_calculate_time = now;
		}
	}


}} // namespace oiran::linda.
