#include <GL/gl.h>
#include <cctype>
#include <fstream>
#include <sstream>
#include <stdexcept>
#include <string>
#include <vector>
#include "display_list.hpp"
#include "gl.hpp"
#include "simple_font.hpp"
#ifndef NDEBUG
#include <iostream>
#endif

namespace oiran { namespace linda {


	namespace {


		void
		throw_exception(const char* file_name, const char* message)
		{
			std::ostringstream oss;
			oss << file_name << ": " << message;
			throw std::runtime_error(oss.str());
		}


	} // unnamed namespace.


	simple_font_t::simple_font_t() :
			M_width(0),
			M_height(0),
			M_draw_width(0),
			M_draw_height(0)
	{
	}

	void
	simple_font_t::load(const char* file_name)
	{
		std::ifstream in(file_name);
		if(!in)
			throw_exception(file_name, "failed to load.");

		std::string line;
		while(line.empty())
			if(!std::getline(in, line))
				break;
		if(in.eof() || line.empty())
			throw_exception(file_name, "invalid format.");

		int w, h;
		std::istringstream iss(line);
		iss >> w >> h;
		if(iss.fail() || w <= 0 || h <= 0)
			throw_exception(file_name, "invalid format.");

		for(std::size_t i = 0; i < number_of_letters; ++i)
			_load_letter(in, w, h, i);

		check_gl_error();

		M_width = w;
		M_height = h;
		M_draw_width = w;
		M_draw_height = h;

#ifndef NDEBUG
		std::cout << "loaded simple font : " << file_name << std::endl;
#endif
	}

	void
	simple_font_t::_load_letter(
			std::ifstream& in, int w, int h, std::size_t index)
	{
		std::vector<int> data(w * h);

		std::string line;
		for(int i = 0; i < h;)
		{
			if(std::getline(in, line))
				if(line.empty())
					continue;
			for(int j = 0; j < w; ++j)
			{
				if(line.size() < static_cast<std::size_t>(j))
					data[i * w + j] = 0;
				else
					data[i * w + j] = (line[j] == '-' ? 0 : 1);
			}
			++i;
		}

		M_letters[index].begin();
		for(int y = 0; y < h; ++y)
			for(int x = 0; x < w; ++x)
				if(data[y * w + x])
					glRecti(x, y, x + 1, y + 1);
		M_letters[index].end();
	}

	void
	simple_font_t::_print(double x, double y, char a)
	{
		if(!std::isprint(a) || a == ' ')
			return;

		const int index = a - (' ' + 1);

		glPushMatrix();
		glTranslated(x, y, 0);
		M_letters[index].call();
		glPopMatrix();
	}

	void
	simple_font_t::size(double w, double h)
	{
		M_draw_width = w;
		M_draw_height = h;
	}


}} // namespace oiran::linda.
