#ifndef OIRAN_LINDA_SIMPLE_FONT_HPP
#define OIRAN_LINDA_SIMPLE_FONT_HPP

#include <GL/gl.h>
#include <cstddef>
#include <fstream>
#include <sstream>
#include <string>
#include <oiran/noncopyable.hpp>
#include <oiran/linda/display_list.hpp>

namespace oiran { namespace linda {


	// simple font.
	/*
	 * this class can draw ASCII[0x21-0x7e] font.
	 *
	 * font data example:
	 * 6 6
	 *
	 * ------
	 * --#---
	 * --#---
	 * --#---
	 * ------
	 * --#---
	 *
	 * ------
	 * -#-#--
	 * -#-#--
	 * ------
	 * ------
	 * ------
	 * ------
	 *
	 * ...
	 */
	class simple_font_t
	{
	public:
		enum { number_of_letters = 0x7e - 0x20 };

		simple_font_t();

		void load(const char* file_name);
		void size(double width, double height);

		template <class T>
		void print(double x, double y, const T& value);

		double width() const { return M_draw_width; }
		double height() const { return M_draw_height; }

	private:
		OIRAN_NONCOPYABLE(simple_font_t);

		void _load_letter(std::ifstream& in, int w, int h, std::size_t index);
		void _print(double x, double y, char a);

	private:
		display_list_t M_letters[number_of_letters];
		int M_width;
		int M_height;
		double M_draw_width;
		double M_draw_height;
	};

	template <class T>
	void
	simple_font_t::print(double x, double y, const T& value)
	{
		if(M_width <= 0 || M_height <= 0 ||
				M_draw_width <= 0 || M_draw_height <= 0)
			return;

		std::string text;
		{
			std::ostringstream oss;
			oss << value;
			text = oss.str();
		}

		const double w = M_draw_width / M_width;
		const double h = M_draw_height / M_height;
		glPushMatrix();
		glTranslated(x, y, 0);
		glScaled(w, h, 0);
		for(std::string::size_type i = 0; i < text.length(); ++i)
			_print(i * M_width, 0, text[i]);
		glPopMatrix();
	}


}} // namespace oiran::linda.

#endif // OIRAN_LINDA_LETTER_HPP
