#include <ctime>
#include <oiran/random.hpp>
#include "resource.hpp"
#include "scene.hpp"


scene_game_t::scene_game_t()
{
	oiran::linda::joystick_t& stick = resource_t::instance()->stick;

	// start recording replay.
	M_out.open(last_relay_file, std::ios::binary);
	std::time_t now = std::time(0);
	M_out.write(reinterpret_cast<const char*>(&now), sizeof(now));
	stick.start_record(M_out);

	oiran::random_seed(now);
	M_ttrs.init(&resource_t::instance()->stick);
}

void
scene_game_t::update()
{
	oiran::linda::joystick_t& stick = resource_t::instance()->stick;

	if(M_ttrs.is_game_over() && M_out.is_open())
	{
		stick.stop_record();
		M_out.close();

		ranking_t& ranking = resource_t::instance()->ranking;
		ranking.update(M_ttrs.score());
	}

	M_ttrs.update();

	if(M_ttrs.is_game_over() && stick.is_pushed(stick.b_button))
		valid(false);
}

void
scene_game_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	M_ttrs.draw(g);

	if(M_ttrs.is_game_over())
	{
		g.color(colors[0]);
		font.size(6, 12);
		font.print(1, height - 13, "push b button to return");
	}
}
