#include <algorithm>
#include <sstream>
#include "defs.hpp"
#include "resource.hpp"
#include "scene.hpp"


const int scene_replay_t::max_speed = 9;


scene_replay_t::scene_replay_t(int index) :
		M_speed(1)
{
	std::ostringstream oss;
	oss << index << ".dat";

	M_replay.init(oss.str().c_str());
}

void
scene_replay_t::update()
{
	oiran::linda::joystick_t& stick = resource_t::instance()->stick;

	stick.update();

	if(stick.is_pushed(stick.up))
		M_speed = std::min(max_speed, M_speed + 1);

	if(stick.is_pushed(stick.down))
		M_speed = std::max(1, M_speed - 1);

	if(stick.is_pushed(stick.b_button))
		valid(false);

	if(M_replay.can_replay())
		for(int i = 0; i < M_speed; ++i)
			M_replay.update();
}

void
scene_replay_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	if(M_replay.can_replay())
		M_replay.draw(g);

	g.color(colors[0], 0.5);
	font.size(24, 32);
	font.print(1, 1, "REPLAY");

	font.size(12, 24);
	font.print(width - 24, 1, "x");
	font.print(width - 12, 1, M_speed);

	g.color(colors[0], 0.5);
	font.size(6, 12);
	font.print(1, height - 13, "push b button to return");
}
