#include "defs.hpp"
#include "resource.hpp"
#include "scene.hpp"


scene_title_t::scene_title_t(scene_manager_t* manager) :
		M_manager(manager),
		M_select(0),
		M_state(demo),
		M_state_count(0),
		M_title_alpha(0),
		M_menu_alpha(0),
		M_ttrs_alpha(1)
{
}

void
scene_title_t::update()
{
	oiran::linda::joystick_t& stick = resource_t::instance()->stick;
	stick.update();

	switch(M_state)
	{
	case demo:
		++M_state_count;

		if(M_state_count < framerate)
			M_title_alpha += 1.0 / framerate;
		else if(M_state_count < framerate * 2)
		{
			M_title_alpha = 1.0;
			M_menu_alpha += 1.0 / framerate;
		}
		else
		{
			M_title_alpha = 1.0;
			M_menu_alpha = 1.0;
		}

		if(M_state_count > framerate * 3 || stick.is_pushed(stick.a_button))
		{
			M_replay.init(last_relay_file);
			M_alphas[0] = 1;
			M_alphas[1] = 0.5;
			M_state = on_select;
			M_state_count = 0;
		}
		break;

	case on_select:
		if(M_state_count < framerate)
		{
			++M_state_count;
			M_ttrs_alpha -= 0.5 / framerate;
		}
		else
			M_ttrs_alpha = 0.5;

		if(stick.is_pushed(stick.up))
		{
			M_select = 0;
			M_alphas[0] = 1;
		}

		if(stick.is_pushed(stick.down))
		{
			M_select = 1;
			M_alphas[1] = 1;
		}

		if(stick.is_pushed(stick.a_button))
		{
			M_manager->push(scene_title);
			M_manager->push(scene_ranking);
			if(M_select == 0)
				M_manager->push(scene_game);
			valid(false);
		}

		if(M_replay.can_replay())
		{
			M_replay.update();
			if(M_replay.is_end())
				M_replay.init(last_relay_file);
		}

		for(int i = 0; i < 2; ++i)
			if(i != M_select && M_alphas[i] > 0.5)
				M_alphas[i] -= 0.05;
	}
}

void
scene_title_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	switch(M_state)
	{
	case demo:
		g.color(colors[0], M_title_alpha);
		font.size(32, 48);
		font.print(1, 1, "TTRS");

		font.size(12, 24);
		g.color(colors[0], M_menu_alpha);
		font.print(120, 192, "GAME START");
		g.color(colors[0], M_menu_alpha);
		font.print(120, 216, "RANKING");
		break;

	case on_select:
		if(M_replay.can_replay())
		{
			M_replay.draw(g);
			g.color(background_color, M_ttrs_alpha);
			g.rect(0, 0, width, height);
		}

		g.color(colors[0]);
		font.size(32, 48);
		font.print(1, 1, "TTRS");

		font.size(12, 24);
		g.color(colors[0], M_alphas[0]);
		font.print(120, 192, "GAME START");
		g.color(colors[0], M_alphas[1]);
		font.print(120, 216, "RANKING");
		break;
	}
}
