#include <GL/gl.h>
#include <assert.h>
#include "resource.hpp"
#include "ttrs.hpp"


ttrs_t::ttrs_t() :
		M_input(0),
		M_state(on_play),
		M_state_count(0)
{
}

void
ttrs_t::init(oiran::linda::button_input_t* input)
{
	assert(input);

	M_input = input;
	M_mino.init(input, &M_field);
	M_field.init(&M_mino);

	M_state = start_wait;
	M_state_count = 0;
}

void
ttrs_t::update()
{
	M_input->update();

	switch(M_state)
	{
	case start_wait:
		if(++M_state_count >= framerate * 3)
		{
			M_state = on_play;
			M_state_count = 0;
			M_mino.entry();
		}
		break;

	case on_play:
		if(M_state_count < framerate)
			++M_state_count;
		if(M_mino.is_game_over() || M_mino.is_game_clear())
		{
			M_state = game_over_wait;
			M_state_count = 0;
		}
		M_mino.update();
		M_field.update();
		break;

	case game_over_wait:
		if(++M_state_count >= framerate)
			M_state = game_over;
		// through.

	case game_over:
		M_mino.update();
		M_field.update();
		break;
	}
}

void
ttrs_t::draw(oiran::linda::graphics_t& g)
{
	oiran::linda::simple_font_t& font = resource_t::instance()->font;

	switch(M_state)
	{
	case start_wait:
		g.color(colors[0]);
		font.size(24, 32);
		if(M_state_count < framerate)
			font.print(48, 100, 3);
		else if(M_state_count < framerate * 2)
			font.print(48, 100, 2);
		else if(M_state_count < framerate * 3)
			font.print(48, 100, 1);
		break;
	case on_play:
		g.color(colors[0]);
		font.size(24, 32);
		if(M_state_count < framerate)
			font.print(36, 100, "GO");
		break;
	}

	M_field.draw(g);
	M_mino.draw(g);

	if(M_mino.is_game_over())
	{
		g.color(0x888888);
		glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
		g.rect(10, 40,
				field_width * block_size, (field_height - 1) * block_size);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}

	g.color(0xffffff);
	font.size(12, 24);
	font.print(120, 40, "LEVEL");
	font.print(120, 64, M_mino.level());

	font.print(120, 100, "SCORE");
	font.print(120, 124, M_field.score());
}
